//BLEMInimal 13/06/2024 11:19:56

char rev[10] = "Rev:A3";    // edit this to match

#include <ArduinoBLE.h>

#include "Arduino_LED_Matrix.h"
ArduinoLEDMatrix matrix;
#include "frames.h"

//UUIDs must match AppInventor!!! sgc use long UUIDs here

//https://www.professorcad.co.uk/appinventortips#BleUuids
#define BLE_SERVICE_UUID "6b18ab86-7622-438c-a712-86a61eccbc7a"
#define BLE_CHARACTERISTIC_UUID "0a81e003-4280-4638-bf38-196f30f2a6df"

//Service
BLEService carService(BLE_SERVICE_UUID);

//Characteristic
BLEShortCharacteristic carControlCharacteristic(BLE_CHARACTERISTIC_UUID, BLEWrite | BLERead | BLENotify, 16);

// --------------------------------------------------------------------------
void setup()
{
         char msg[30] = "";

         // 1. Serial
         Serial.begin(115200);
         while(! Serial);
         Serial.println();
         Serial.println("*** Start...");

           // 2. LED Matrix on R4 board
           matrix.begin(); // LED Matrix
           matrix.loadFrame(stop);

           // 3. BLE
           if (!BLE.begin())
           {
                  Serial.println("starting Bluetooth Low Energy module failed!");
                  while (1)
                  { // blink the built-in LED fast to indicate an issue
                    digitalWrite(LED_BUILTIN, HIGH);
                    delay(100);
                    digitalWrite(LED_BUILTIN, LOW);
                    delay(100);
                  }
           }

         // Set up BLE
         BLE.setLocalName("UnoR4BLE");
         BLE.setAdvertisedService(carService);
         carService.addCharacteristic(carControlCharacteristic); // add the characteristics to the service
         BLE.addService(carService); // add the service
         carControlCharacteristic.writeValue(0);
         BLE.advertise(); // start advertising
         Serial.println("Bluetooth Low Energy device active, waiting for connections...");
}

// --------------------------------------------------------------------------
void loop()
{
        short AppCmd; //Buffer

        blinkLedNoDelay();

        BLEDevice controller = BLE.central(); // listen for BLE peripherals to connect (i.e. smartphone):

        if (controller)  // if a central is connected to peripheral:
        {
               Serial.print("Connected to controller: ");
               Serial.println(controller.address()); // print the controller's MAC address:
               matrix.loadFrame(happy);

               // while the controller is still connected to peripheral:
               while (controller.connected())
               {
                      carControlCharacteristic.readValue(AppCmd, 16)

                      if (AppCmd > 0)
                      {
                        switch (AppCmd) {
                          case 1:
                            Serial.println("LEFT");
                            matrix.loadFrame(arrow_left);
                            break;

                          case 2:
                            Serial.println("RIGHT");
                            matrix.loadFrame(arrow_right);
                            break;

                          case 3:
                            Serial.println("UP");
                            matrix.loadFrame(arrow_up);
                            break;

                          case 4:
                            Serial.println("DOWN");
                            matrix.loadFrame(arrow_down);
                            break;

                          default:  // 0 or invalid control
                            Serial.println("STOP");
                            matrix.loadFrame(stop);
                            break;
                        }
                      }
               }

               // when the central disconnects, print it out:
               Serial.print(F("Disconnected from controller: "));
               Serial.println(controller.address());
               //digitalWrite(LED_BUILTIN, LOW);         // when the central disconnects, turn off the LED
               matrix.loadFrame(stop);
    }
}

//---------------------------------------------------------------
// reference: https://www.baldengineer.com/arduino-how-do-you-reset-millis.html
void blinkLedNoDelay(void)  // Blink the LED without using Delay in the main loop
{
  static long now;
  static const int period = 1000; // millisecs
  static long lastChangeTime = 0;

  // blink LED without delay in loop
  now = millis();                       // Rolls over in ~49 days
  //if(now > (lastChangeTime + period)) // Bug! Doesn't roll over correctly!
  if((now - lastChangeTime) >= period)  // ---FIXED---
  {
    if(digitalRead(LED_BUILTIN) == HIGH)
    {
      digitalWrite(LED_BUILTIN, LOW);
      //matrix.loadFrame(arrow_up);
    }
    else
    {
      digitalWrite(LED_BUILTIN, HIGH);
      //matrix.loadFrame(arrow_down);
    }
    lastChangeTime = now;
  }
}
